/*************************************************
*     Exim - an Internet mail transport agent    *
*************************************************/

/* Copyright (c) University of Cambridge 1995 - 2015 */
/* See the file NOTICE for conditions of use and distribution. */

/* Header for eldapauth_find */

extern int     eldapauth_find(void *, uschar *, const uschar *, int, uschar **,
                 uschar **, BOOL *);

/* End of lookups/ldap.h */
